@@ -30,7 +30,7 @@ from itertools import *
 import operator
 import traceback
 from contextlib import contextmanager
-from types import StringTypes
+from types import StringTypes, GeneratorType
 from calculate.lib.configparser import ConfigParser, ParsingError
 
 def addStdConfig(fileLst,prefix='/'):
@@ -80,7 +80,44 @@ setLocalTranslate('cl_lib3',sys.modules[__name__])
 READONLY = "r"
 WRITEABLE = "w"
 
-class Variable:
+class VariableInterface(object):
+    def Get(self, varname=None, humanreadable=False):
+        pass
+
+    def Choice(self, varname):
+        pass
+
+    def Select(self, selField, **kwargs):
+        pass
+
+    def select(self, *fields, **kw):
+        pass
+
+    def ZipVars(self, *argvVarNames):
+        pass
+
+    def uncompatible(self):
+        pass
+
+    def set(self, value):
+        pass
+
+    def get(self):
+        pass
+
+    def choice(self):
+        pass
+
+    def GetBool(self, varname):
+        pass
+
+    def GetInteger(self, varname):
+        pass
+
+    def invalidate(self,force=False):
+        pass
+
+class Variable(VariableInterface):
     """
     Class of variable. Need for creating calculate utilities variable
     add fill method described by class with fill methods.
@@ -346,6 +383,8 @@ class Variable:
         Convert choice created by variable choice to two tuple (values,comments)
         """
         res = self.choice()
+        if isinstance(res, GeneratorType):
+            res = tuple(res)
         if res and type(res[0]) in (tuple,list):
             return zip(*res)
         else:
@@ -430,6 +469,9 @@ class Variable:
             return
         if "choice" in self.type:
             choiceVal = self.choice()
+            if isinstance(choiceVal, GeneratorType):
+                choiceVal = tuple(choiceVal)
+
             tipChoice = map(lambda x:'"%s" [%s]'%(x[1],x[0]) \
                             if type(x) in (list,tuple) else str(x), choiceVal)
             if choiceVal and type(choiceVal[0]) in (tuple,list):
@@ -713,7 +755,7 @@ class ActionVariable(ReadonlyVariable):
             return "off"
         return self.action(self.Get('cl_action'))
 
-class FieldValue:
+class FieldValue(VariableInterface):
     """
     Table column variable
     """
@@ -1121,8 +1163,8 @@ class DataVars(SimpleDataVars):
                              not "Variable" in x,
                       dir(varModule)))+[varModule]:
             for classname in \
-                    ifilterfalse(("ReadonlyVariable",
-                                 "Variable","VariableError").__contains__,
+                    ifilterfalse(("ReadonlyVariable", "VariableInterface",
+                                  "Variable", "VariableError").__contains__,
                     ifilter(lambda x:x.startswith("Variable"),
                     dir(varMod))):
                 varObj = getattr(varMod,classname)
