@@ -1,6 +1,6 @@
 #-*- coding: utf-8 -*-
 
-# Copyright 2008-2013 Calculate Ltd. http://www.calculate-linux.org
+# Copyright 2008-2015 Calculate Ltd. http://www.calculate-linux.org
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -21,6 +21,9 @@ from os import path
 import time
 import datetime
 import xml
+import functools
+import bz2
+
 if hasattr(xml,"use_pyxml"):
     xml.use_pyxml()
 from xml import xpath
@@ -28,19 +31,18 @@ import xml.etree.ElementTree as ET
 import pexpect
 from calculate.lib.configparser import ConfigParser
 
-from calculate.lib.cl_lang import setLocalTranslate, getLazyLocalTranslate
 from calculate.lib.utils.colortext.palette import TextState
 from calculate.lib.utils.common import cmpVersion
-from calculate.lib.utils.files import (getProgPath, STDOUT, find,
-                                       PercentProgress, process, readFile,
-                                       readLinesFile)
-from calculate.lib.utils.tools import AddonError, SavableIterator, ignore
+from calculate.lib.utils.files import (getProgPath, find, process,
+                                       readLinesFile, pathJoin)
+from calculate.lib.utils.tools import SavableIterator, ignore
 from collections import Mapping, defaultdict
 from common import getTupleVersion
 from contextlib import closing
 from files import listDirectory, readFile
 from functools import total_ordering
 from itertools import ifilter, imap, chain
+from git import Git, GitError
 Colors = TextState.Colors
 import glob
 
@@ -51,7 +53,7 @@ reVerSplit = re.compile(r"^(?:.*/var/db/pkg/)?(?:(\w+-\w+)/)?(.*?)-(([^-]+?)"
                          "(?:-(r\d+))?)(?:.(tbz2))?$",re.S)
 def reVerSplitToPV(x):
     """Convert match from reVerSplit to PV hash"""
-    if type(x) == str:
+    if type(x) in (str, unicode):
         x = reVerSplit.search(x)
     if x:
         match = x.groups()
@@ -143,27 +145,68 @@ def getSquashList():
             gzipVer = getTupleVersion('4.2')
             if pkgVer >= gzipVer:
                 usesSquashFs.append('gzip')
-        return map(lambda x:{"lzma":"xz"}.get(x,x),
-               list(set(usesSquashFs) & wantMethod))
+        return list(set(usesSquashFs) & wantMethod)
 
-def searchProfile(dirpath,configname):
+
+class RepositorySubstituting(object):
+    """
+    Объект выполняющий подставнку repository:path пути
+
+    так как для подстановки необходимо получить все пути репозиториев, и это
+    занимает время, то объек извлекает данные из переменной только при
+    необходимости: в строке найдено repository:
+    """
+    token = re.compile("^\w+:")
+
+    def __init__(self, dv, system_root=''):
+        self.dv = dv
+        self._substitution = None
+        self.system_root = system_root.rstrip("/")
+
+    @classmethod
+    def has_repos_name(cls, s):
+        return bool(cls.token.search(s))
+
+    def _prepare_substitution(self):
+        # TODO: проверка выхода за chroot
+        emerge_config = self.dv.Get('cl_emerge_config')
+        if emerge_config and emerge_config.repositories:
+            repos = {x.name.encode('utf-8'): x.location.encode('utf-8')
+                     for x in emerge_config.repositories }
+            r = re.compile("|".join("^%s:" % x for x in repos.keys()))
+            self._substitution = (
+                functools.partial(
+                    r.sub, lambda *args: "%s%s/profiles/" % (
+                        self.system_root, repos.get(
+                            args[0].group(0)[:-1], ""))))
+        else:
+            self._substitution = lambda x:x
+
+    def __call__(self, s):
+        if self.has_repos_name(s):
+            if not self._substitution:
+                self._prepare_substitution()
+            return self._substitution(s)
+        return s
+
+
+def searchProfile(dirpath, configname, repository_sub=None):
     """Search profile"""
-    paths = []
 
     def search(dirpath):
         parentpath = path.join(dirpath, "parent")
         if path.exists(parentpath):
             for line in open(parentpath, 'r'):
-                search(path.join(dirpath, line.strip()))
+                if repository_sub:
+                    line = repository_sub(line)
+                search_dn = path.realpath(path.join(dirpath, line.strip()))
+                for dn in search(search_dn):
+                    yield dn
         fullconfig = path.join(dirpath, configname)
         if path.exists(fullconfig):
-            paths.append(fullconfig)
+            yield fullconfig
 
-    search(dirpath)
-    return paths
-
-class GitError(AddonError):
-    """Git Error"""
+    return list(search(dirpath))
 
 class Layman:
     """
@@ -177,10 +220,11 @@ class Layman:
     layman_package = "app-portage/layman"
     portage_package = "sys-apps/portage"
 
-    def __init__(self, installed, makeconf, laymanconf=None):
+    def __init__(self, installed, makeconf, laymanconf=None, prefix="/"):
         self.installed = installed
         self.makeconf = makeconf
         self.laymanconf = laymanconf
+        self.prefix = prefix
 
     def _add_to_laymanconf(self, rname, rurl, rpath):
         if not self.laymanconf:
@@ -198,7 +242,17 @@ class Layman:
             config.set(rname, k, v)
         with open(self.laymanconf,'wb') as f:
             config.write(f)
-            
+
+    def _remove_from_laymanconf(self, rname):
+        if not self.laymanconf:
+            return
+        config = ConfigParser(strict=False)
+        config.read(self.laymanconf, encoding="utf-8")
+        if config.has_section(rname):
+            config.remove_section(rname)
+            with open(self.laymanconf, 'wb') as f:
+                config.write(f)
+
     def _add_to_installed(self, rname, rurl):
         """
         Добавить репозиторий в installed.xml
@@ -257,21 +311,79 @@ class Layman:
         with open(self.makeconf, 'w') as f:
             f.write(content)
 
+    def _remove_from_makeconf(self, rpath):
+        """
+        Удалить путь из make.conf
+        """
+        def fixContent(match):
+            repos = match.group(1).strip().split('\n')
+            if rpath in repos:
+                repos.remove(rpath)
+            return 'PORTDIR_OVERLAY="\n%s"' % "\n".join(repos);
+
+        if path.exists(self.makeconf):
+            content = readFile(self.makeconf)
+            if "PORTDIR_OVERLAY" in content:
+                new_content = re.sub(
+                    "\APORTDIR_OVERLAY=\"([^\"]+)\"",
+                    fixContent, content, re.DOTALL)
+                if new_content == content:
+                    return
+                with open(self.makeconf, 'w') as f:
+                    f.write(new_content)
+
+    def is_new_layman(self):
+        layman_pkg_info = isPkgInstalled(self.layman_package,
+                                         prefix=self.prefix.encode('utf-8'))
+        portage_pkg_info = isPkgInstalled(self.portage_package,
+                                          prefix=self.prefix.encode('utf-8'))
+        if layman_pkg_info and portage_pkg_info:
+            layman_ver = layman_pkg_info[0]['PV']
+            portage_ver = portage_pkg_info[0]['PV']
+            if (cmpVersion(layman_ver, "2.3") >= 0 and
+                        cmpVersion(portage_ver, "2.2.18") >= 0):
+                return True
+        return False
+
     def add(self, rname, rurl, rpath):
         """
         Добавить репозиторий в installed.xml и layman/make.conf
         """
         self._add_to_installed(rname, rurl)
-        layman_pkg_info = isPkgInstalled(self.layman_package)
-        portage_pkg_info = isPkgInstalled(self.portage_package)
-        if layman_pkg_info and portage_pkg_info:
-            layman_ver = layman_pkg_info[0]['PV']
-            portage_ver = portage_pkg_info[0]['PV']
-            if (cmpVersion(layman_ver, "2.3") >= 0 and 
-                cmpVersion(portage_ver, "2.2.18") >= 0):
-                self._add_to_laymanconf(rname, rurl, rpath)
-                return True
-        self._add_to_makeconf(rpath)
+        if self.is_new_layman():
+            self._add_to_laymanconf(rname, rurl, rpath)
+        else:
+            self._add_to_makeconf(rpath)
+        return True
+
+    def _remove_from_installed(self, rname):
+        """
+        Удалить репозиторий в installed.xml
+        """
+        if path.exists(self.installed) and readFile(self.installed).strip():
+            tree = ET.parse(self.installed)
+            root = tree.getroot()
+            el = root.find("repo[name='%s']" % rname)
+            if el is not None:
+                root.remove(el)
+                try:
+                    from layman.utils import indent
+                    indent(root)
+                except ImportError as e:
+                    pass
+                with open(self.installed, 'w') as f:
+                    f.write('<?xml version="1.0" encoding="UTF-8"?>\n')
+                    tree.write(f, encoding="utf-8")
+
+    def remove(self, rname, rpath):
+        """
+        Удалить репозиторий из настроек layman
+        """
+        self._remove_from_installed(rname)
+        if self.is_new_layman():
+            self._remove_from_laymanconf(rname)
+        else:
+            self._remove_from_makeconf(rpath)
         return True
 
     def get_installed(self):
@@ -284,352 +396,6 @@ class Layman:
         return []
 
 
-class Git:
-    """
-    Объект для управление git репозиторием
-    """
-    skip_files = ["metadata/md5-cache",
-                  "metadata/cache"]
-
-    def __init__(self):
-        self._git = self.__getGit()
-
-    def checkExistsRep(self, rpath):
-        """
-        Проверить путь на наличие репозитория
-        """
-        if path.exists(rpath):
-            if not path.isdir(rpath):
-                raise GitError(
-                    _("Repository {path} is not a directory").format(
-                        path=rpath))
-            if not path.isdir(self._gitDir(rpath)):
-                raise GitError(
-                    _("Repository {path} is not Git").format(
-                        path=rpath))
-            return True
-        return False
-
-    def __getGit(self):
-        """
-        Получить утилиту git
-        """
-        git = getProgPath("/usr/bin/git")
-        if not git:
-            raise GitError(_("The Git tool is not found"))
-        return git
-
-    @staticmethod
-    def _gitDir(rpath):
-        return path.join(rpath, ".git")
-
-    @staticmethod
-    def is_git(gitpath):
-        return path.isdir(Git._gitDir(gitpath))
-
-    def get_url(self, rpath, remote_name):
-        config = ConfigParser(strict=False)
-        config.read(path.join(Git._gitDir(rpath), "config"), encoding="utf-8")
-        return config.get('remote "%s"'%remote_name, "url",
-                          raw=True, fallback="")
-
-    def cloneRepository(self, url, rpath, branch, cb_progress=None):
-        """
-        Сделать локальную копию репозитория
-
-        Args:
-            url: откуда качать репозиторий
-            rpath: куда сохранять репозиторий
-            branch: ветка на которую необходимо переключиться
-        """
-        if cb_progress:
-            gitClone = PercentProgress(self._git, "clone", "-q",
-                                       "--no-single-branch", "--progress",
-                                       "--verbose",
-                                       "--depth=1", "-b", branch, url, rpath,
-                                       part=4, stderr=STDOUT)
-            for perc in gitClone.progress():
-                cb_progress(perc)
-        else:
-            gitClone = process(self._git, "clone", "-q", "--no-single-branch",
-                               "--depth=1", "-b", branch, url, rpath,
-                               stderr=STDOUT)
-        if gitClone.failed():
-            error = gitClone.read()
-            if "Remote branch %s not found" % branch in error:
-                raise GitError(
-                    _("Branch {branch} not found in repository {url}").format(
-                        branch=branch, url=url))
-            raise GitError(_("Failed to clone repository {url}").format(
-                url=url), error)
-        return True
-
-    def cloneRevRepository(self, url, rpath, branch, revision,
-                           cb_progress=None):
-        """
-        Сделать локальную копию репозитория с указанной ревизией
-
-        Args:
-            url: откуда качать репозиторий
-            rpath: куда сохранять репозиторий
-            branch: ветка на которую необходимо переключиться
-            revision: если указана - сделать ревизию текущей
-
-        Return:
-            Возвращает True если клонирование произведено с установкой на
-            указанную ревизию. False если клонирование произведено с
-            установкой на последнюю ревизию.
-        Raises:
-            GitError: Выполнение ключевых команд выполнено с ошибками (не
-                удалось скачать и получить данные из удаленного репозитория)
-        """
-        git_dir = self._gitDir(rpath)
-        error = []
-
-        def command(cmd, startpart=0):
-            """
-            Выполнить одну из команд необходимой для клонирования репозитория
-            """
-            commands = {  # инициализация пустого репозитория
-                          'init': ["init", rpath],
-                          # подключить указанный удаленный как origin
-                          'add_remote': ["remote", "add", "origin", url],
-                          # скачать последние коммиты веток
-                          'fetchshallow': ["fetch", "--depth=1"],
-                          # проверить есть указанный коммит
-                          'has_revision': ["log", "-n1", revision],
-                          # проверить есть указанный коммит
-                          'has_branch': ["log", "-n1",
-                                         "remotes/origin/%s" % branch],
-                          # получить ревизию из revs тэгов
-                          'get_rev_tag': ["fetch", "--depth=1", "origin",
-                                          "+refs/revs/%s:refs/remotes/origin/%s" %
-                                          (revision, branch)],
-                          # переключиться на указанную ревизию указанной веткой
-                          'checkout_revision': ["checkout", "-b", branch,
-                                                revision],
-                          # переключить на указанную ветку
-                          'checkout': ["checkout", branch],
-                          # установить upstream для локальной ветки
-                          'set_track': ["branch", branch, '-u',
-                                        "origin/%s" % branch]
-            }
-            if cmd == "init":
-                wholeCommand = [self._git] + commands[cmd]
-            else:
-                wholeCommand = [self._git, "--git-dir", git_dir,
-                                "--work-tree", rpath] + commands[cmd]
-            if cb_progress and commands[cmd][0] in ("fetch", "checkout"):
-                progressParam = {'fetch': {'part': 4, 'end': False},
-                                 'checkout': {'part': 4, 'startpart': 3}}
-                gitClone = PercentProgress(
-                    *wholeCommand + ["--progress", "--verbose"],
-                    stderr=STDOUT, **progressParam)
-                for perc in gitClone.progress():
-                    cb_progress(perc)
-            else:
-                gitCmd = process(*wholeCommand, stderr=STDOUT)
-            if gitCmd.failed():
-                error.append(gitCmd.read())
-                return False
-            return True
-
-        # получить последние коммиты из удаленного репозитория
-        if command("init") and command("add_remote"):
-            if command("get_rev_tag") or command("fetchshallow"):
-                if not command("has_branch"):
-                    raise GitError(
-                        _("Branch {branch} not found in repository {url}"
-                        ).format(branch=branch, url=url))
-                # если среди коммитов есть указанный коммит
-                if command("has_revision"):
-                    # переключаемся на нужный коммита, устанавливаем связь
-                    if command("checkout_revision") and command("set_track"):
-                        return True
-                elif command("checkout"):
-                    return False
-        raise GitError(_("Failed to clone repository {url}").format(
-            url=url), error[-1])
-
-    def pullRepository(self, rpath, quiet_error=False, cb_progress=None):
-        """
-        Обновить репозиторий до последней версии
-        """
-        gitPull = process(self._git, "--git-dir", self._gitDir(rpath),
-                          "pull", "--ff-only", stderr=STDOUT)
-        if gitPull.failed():
-            if not quiet_error:
-                error = gitPull.read()
-                raise GitError(
-                    _("Failed to update the repository in {rpath}").format(
-                        rpath=rpath), error)
-            return False
-        return True
-
-    def fetchRepository(self, rpath, cb_progress=None):
-        """
-        Получить изменения из удаленно репозитория
-        """
-        if cb_progress:
-            gitFetch = PercentProgress(self._git, "--git-dir",
-                                       self._gitDir(rpath),
-                                       "fetch", "--progress", "--verbose",
-                                       part=3, stderr=STDOUT)
-            for perc in gitFetch.progress():
-                cb_progress(perc)
-        else:
-            gitFetch = process(self._git, "--git-dir", self._gitDir(rpath),
-                               "fetch", stderr=STDOUT)
-        if gitFetch.failed():
-            error = gitFetch.read()
-            raise GitError(
-                _("Failed to update the repository in {rpath}").format(
-                    rpath=rpath), error)
-        return True
-
-    def checkChanges(self, rpath):
-        """
-        Проверить наличие изменений пользователем файлов в репозитории
-        """
-        git_dir = self._gitDir(rpath)
-        git_status = process(self._git, "--git-dir", git_dir, "--work-tree",
-                             rpath,
-                             "status", "--porcelain", stderr=STDOUT)
-        if git_status.success():
-            return not any(x.strip() for x in git_status)
-        else:
-            raise GitError(
-                _("Wrong repository in the {rpath} directory").format(
-                    rpath=rpath))
-
-    def parseStatusInfo(self, data):
-        """
-        Разобрать информацию полученную через git status -b --porcelain
-
-        Returns:
-            Словарь
-            # есть ли измененные файлы пользователем
-            {'files':True/False,
-            # есть коммиты после текущего
-             'ahead':True/False,
-            # есть коммиты перед текущим (означает, что обновление
-            # с основной веткой не осуществляется через fast-forward
-             'behind':True/False,
-            # текущая ветка
-             'branch':'',
-            # оригинальная ветка
-             'origin':'origin/master'}
-        """
-        reStatus = re.compile("^## (\w+)\s*(\(no branch\))?(?:\.\.\.(\S+))?(?:\s+\[(ahead \d+)?"
-                              "(?:, )?(behind \d+)?\])?\n?(.*|$)", re.S)
-        match = reStatus.search(data)
-        if not match:
-            return {}
-        return {'files': True if match.group(6) else False,
-                'ahead': True if match.group(4) else False,
-                'behind': True if match.group(5) else False,
-                'origin': match.group(3) or "",
-                'branch': "" if match.group(2) else match.group(1)}
-
-    def getCurrentCommit(self, rpath):
-        """
-        Получить текущий коммит в репозитории
-        """
-        git_dir = self._gitDir(rpath)
-        git_show = process(self._git, "--git-dir", git_dir, "show",
-                           "--format=format:%H",
-                           "--quiet", stderr=STDOUT)
-        if git_show.success():
-            return git_show.read().strip()
-        else:
-            raise GitError(
-                _("Failed to get the repository status for {rpath}").format(
-                    rpath=rpath))
-
-    def getStatusInfo(self, rpath):
-        """
-        Получить информацию об изменениях в репозитории
-
-        Returns:
-            Словарь выдаваемый функцией _parseStatusInfo
-        """
-        git_dir = self._gitDir(rpath)
-        git_status = process(self._git, "--git-dir", git_dir, "--work-tree",
-                             rpath,
-                             "status", "-b", "--porcelain", stderr=STDOUT)
-        if git_status.success():
-            retDict = self.parseStatusInfo(git_status.read())
-            if not retDict:
-                raise GitError(
-                    _("Failed to get the repository status for {rpath}").format(
-                        rpath=rpath))
-            return retDict
-        else:
-            raise GitError(
-                _("Wrong repository in the {rpath} directory").format(
-                    rpath=rpath))
-
-    def resetRepository(self, rpath, to_origin=False, to_rev=None, info=None):
-        """
-        Удалить неиндексированные изменения в репозитории
-
-
-        Args:
-            to_origin: откатить все изменения до удаленного репозитория
-            to_rev: откатить все изменения до определенной ревизии
-            info: использовать уже полученную информация об изменения в репозитории
-        Return:
-            True - успешное выполнение
-            False - нет необходимости выполнять reset
-        Raises:
-            GitError: выполнение комманд reset и clean прошло с ошибкой
-        """
-        git_dir = self._gitDir(rpath)
-        if not info:
-            info = self.getStatusInfo(rpath)
-        if (all(not info[x] for x in ("files", "ahead", "behind") if x in info)
-            and (not info["origin"] or
-                             "origin/%s" % info["branch"] == info["origin"])):
-            return False
-        commit = (info['origin'] if to_origin else to_rev) or "HEAD"
-        git_reset = process(self._git, "--git-dir", git_dir, "--work-tree",
-                            rpath,
-                            "reset", "--hard", commit, stderr=STDOUT)
-        git_clean = process(self._git, "--git-dir", git_dir, "--work-tree",
-                            rpath,
-                            "clean", "-fd", stderr=STDOUT)
-        if git_reset.failed() or git_clean.failed():
-            raise GitError(_("Failed to clean the {rpath} repository").format(
-                rpath=rpath))
-        return True
-
-    def getBranch(self, rpath):
-        """
-        Получить текущую ветку
-        """
-        return self.getStatusInfo(rpath)['branch']
-
-    def checkoutBranch(self, rpath, branch):
-        """
-        Сменить ветку
-        """
-        git_dir = self._gitDir(rpath)
-        git_checkout = process(self._git, "--git-dir", git_dir,
-                               "--work-tree", rpath,
-                               "checkout", "-f", branch, stderr=STDOUT)
-        if git_checkout.failed():
-            error = git_checkout.read()
-            if "pathspec '%s' did not match" % branch in error:
-                raise GitError(
-                    _("Branch {branch} not found in repository {rpath}").format(
-                        branch=branch, rpath=rpath))
-            raise GitError(
-                _("Failed to move to branch {branch} in the {rpath} "
-                  "repository").format(branch=branch,
-                                               rpath=rpath), error)
-        return True
-
-
 @total_ordering
 class EmergePackage(Mapping):
     """
@@ -893,7 +659,7 @@ class EmergeUpdateInfo(Mapping):
     def __repr__(self):
         return "EmergeUpdateInfo(%s/%s,%s)" % (
             self['CATEGORY'], self['PF'],
-            "binary" if self.binary else "ebuild")
+            "binary" if self['binary'] else "ebuild")
 
     def __str__(self):
         return "%s/%s" % (self['CATEGORY'], self['PF'])
@@ -906,7 +672,7 @@ class EmergeRemoveInfo(EmergeUpdateInfo):
     install_info = "\[(uninstall)([^\]]+)\]"
 
 
-class Eix:
+class Eix(object):
     """
     Вызов eix
 
@@ -920,6 +686,7 @@ class Eix:
         Installed = '--installed'
         Xml = '--xml'
         Upgrade = '--upgrade'
+        TestObsolete = '--test-obsolete'
 
 
     default_options = [Option.Xml]
@@ -941,11 +708,14 @@ class Eix:
             ret = ver.attrib['id']
         yield ret
 
+    def _process(self):
+        return process(self.cmd, *self.options)
+
     def get_output(self):
         """
         Получить вывод eix
         """
-        with closing(process(self.cmd, *self.options)) as p:
+        with closing(self._process()) as p:
             return p.read()
 
     def get_packages(self):
@@ -961,12 +731,15 @@ class Eix:
     def _get_packages(self, et):
         for pkg in et:
             for version in self._get_versions(pkg):
-                yield "%s-%s" % (pkg.attrib['name'], version)
+                if version:
+                    yield "%s-%s" % (pkg.attrib['name'], version)
+                else:
+                    yield pkg.attrib['name']
 
     def _get_categories(self, et):
         for category in et:
             for pkg in self._get_packages(category):
-                yield "%s/%s" % (category.attrib['name'], pkg)
+                yield EmergePackage("%s/%s" % (category.attrib['name'], pkg))
 
     def _parseXml(self, buffer):
         try:
@@ -975,6 +748,21 @@ class Eix:
         except ET.ParseError:
             return iter(())
 
+class ChrootEix(Eix):
+    """
+    Eix выполняемый в chroot
+    """
+
+    def __init__(self, chroot_path, package, *options, **kwargs):
+        self.chroot_cmd = getProgPath("/usr/bin/chroot")
+        self.chroot_path = chroot_path
+        self.cmd = getProgPath("/usr/bin/eix", prefix=chroot_path)
+        super(ChrootEix, self).__init__(package, *options, **kwargs)
+
+    def _process(self):
+        return process(self.chroot_cmd, *([self.chroot_path, self.cmd]
+                                          + self.options))
+
 
 class EmergeLogTask(object):
     def has_marker(self, line):
@@ -1034,18 +822,19 @@ class EmergeLog:
     """
     EmergeLog(after).get_update(package_pattern)
     """
-    emerge_log = "/var/log/emerge.log"
+    _emerge_log = "var/log/emerge.log"
     re_complete_emerge = re.compile(r":::\scompleted (emerge) \(.*?\) (\S+)",
                                     re.M)
     re_complete_unmerge = re.compile(r">>>\s(unmerge) success: (\S+)", re.M)
 
-    def __init__(self, emerge_task=EmergeLogTask()):
+    def __init__(self, emerge_task=EmergeLogTask(), prefix='/'):
         """
         @type emerge_task: EmergeLogTask
         """
         self.emerge_task = emerge_task
         self._list = None
         self._remove_list = None
+        self.emerge_log = path.join(prefix, self._emerge_log)
 
     def _get_last_changes(self):
         """
@@ -1239,3 +1028,119 @@ def get_remove_list(directory, filelist, depth=1):
               fullpath=True, depth=depth)):
         if path.isfile(fn):
             yield fn
+
+class SimpleRepositoryMapper(Mapping):
+    """
+    Определение пути до репозитория
+    """
+    map_repository = {'gentoo': '/usr/portage',
+                      'calculate': '/var/lib/layman/calculate'}
+    layman_path = '/var/lib/layman'
+
+    def __init__(self, prefix='/'):
+        self.prefix = prefix
+
+    def __getitem__(self, item):
+        if item in self.map_repository:
+            return pathJoin(self.prefix, self.map_repository[item])
+        return pathJoin(self.prefix, self.layman_path, item)
+
+    def __iter__(self):
+        return iter(self.map_repository)
+
+    def __len__(self):
+        return len(self.map_repository)
+
+
+class EbuildInfoError(Exception):
+    pass
+
+
+class EbuildInfo(object):
+    """
+    Информация о ebuild (DEPEND) из metadata
+    """
+    meta_suffix_path = 'metadata/md5-cache'
+    support_keys = ('IUSE', 'DEPEND', 'RDEPEND', 'PDEPEND')
+
+    def __init__(self, atom, rpath):
+        meta_path = path.join(rpath,
+                              self.meta_suffix_path)
+        self._meta_info_path = path.join(meta_path, atom)
+        if not path.exists(self._meta_info_path):
+            raise EbuildInfoError("Package is not found")
+        self._info = self._get_info()
+
+    def _get_info(self):
+        with open(self._meta_info_path, 'r') as f:
+            return {k.strip(): v.strip() for k, v in (line.partition('=')[::2]
+                                                      for line in f.readlines())
+                    if k in self.support_keys}
+
+    def __getitem__(self, item):
+        if item in self.support_keys:
+            return self._info.get(item, '')
+        raise KeyError(item)
+
+    def __eq__(self, other):
+        return all(other[k] == self[k] for k in self.support_keys)
+
+    def __ne__(self, other):
+        res = any(other[k] != self[k] for k in self.support_keys)
+        # TODO: DEBUG
+        #if res:
+        #    for k in self.support_keys:
+        #        if other[k] != self[k]:
+        #            print k,":", other[k], "!=", self[k]
+        return res
+
+
+class InstalledPackageInfoError(Exception):
+    pass
+
+
+class InstalledPackageInfo(object):
+    """
+    Информация об установленном пакете (DEPEND) из /var/db/pkg
+    """
+    depend_pattern = 'declare (?:-x )?({0})="([^"]+)"'
+    re_depend = re.compile(depend_pattern.format(
+        "|".join(EbuildInfo.support_keys)), re.DOTALL)
+    re_multispace = re.compile("\s+", re.DOTALL)
+
+    def __init__(self, atom, pkg_dir):
+        self.atom = atom
+        self._pkg_path = path.join(pkg_dir, atom)
+        if not path.exists(self._pkg_path):
+            raise InstalledPackageInfoError("Package is not found")
+        self._info = self._get_info()
+
+    def _get_info(self):
+        info = {k: "" for k in EbuildInfo.support_keys}
+        env_path = path.join(self._pkg_path, 'environment.bz2')
+        if path.exists(env_path):
+            with bz2.BZ2File(env_path, 'r') as f:
+                for r in self.re_depend.finditer(f.read()):
+                    key, value = r.groups()
+                    value = self.re_multispace.sub(" ", value)
+                    info[key] = value.strip()
+        rep_path = path.join(self._pkg_path, 'repository')
+        info['repository'] = readFile(rep_path).strip()
+        return info
+
+    def __getitem__(self, item):
+        return self._info[item]
+
+    @classmethod
+    def get_install_packages(cls, pkg_dir='/var/db/pkg'):
+        for category in listDirectory(pkg_dir):
+            for pkg in listDirectory(path.join(pkg_dir, category)):
+                if "MERGING" not in pkg:
+                    yield InstalledPackageInfo("%s/%s" % (category, pkg),
+                                               pkg_dir=pkg_dir)
+
+    def __str__(self):
+        return self.atom
+
+    def __repr__(self):
+        return "InstalledPackageInfo(%s)" % self.atom
