@@ -18,11 +18,14 @@ import os
 from os import path
 import re
 import platform
-from calculate.lib.datavars import Variable, SimpleDataVars
-from calculate.lib.utils.portage import isPkgInstalled, searchProfile
-from calculate.lib.utils.files import readFile
-from env import VariableClMakeProfile
+from calculate.lib.datavars import (Variable, SimpleDataVars, ReadonlyVariable,
+            VariableError)
+
+from calculate.lib.utils.portage import isPkgInstalled, searchProfile, \
+    RepositorySubstituting
+from calculate.lib.utils.files import readFile, pathJoin
 from calculate.lib.configparser import ConfigParser
+import env
 
 class Linux:
     dictLinuxName = {"CLD":"Calculate Linux Desktop",
@@ -220,7 +223,7 @@ class VariableOsLinuxFiles(Variable,Linux):
     def get(self):
         """Get files count. Count files discard because it to long
         operation. This value is already count by calculate-builder and
-        is placed in calculate3.env"""
+        is placed in calculate.env"""
         return self.Get('os_linux_filesnum')
         #return str(countFiles(self.systemRoot))
 
@@ -229,7 +232,9 @@ class VariableOsLinuxPkglist(Variable):
     type = "list"
 
     def generate_shortnames(self, make_profile):
-        for fn in searchProfile(make_profile, "calculate.env"):
+        repos = RepositorySubstituting(self)
+        for fn in searchProfile(make_profile, "calculate.env",
+                                repository_sub=repos):
             config = ConfigParser(strict=False)
             config.read(fn, encoding="utf-8")
             value = SimpleDataVars.unserialize("string",
@@ -246,25 +251,86 @@ class VariableOsLinuxPkglist(Variable):
             return []
 
 
+class VariableClProfileSystem(ReadonlyVariable):
+    """
+    Профиль системы (симлинк /etc/make.profile')
+    """
+    def get(self):
+        try:
+            make_profile = self.Get('cl_make_profile')
+            return path.normpath(
+                path.join(path.dirname(make_profile),
+                          os.readlink(make_profile)))
+        except:
+            raise VariableError(_("Failed to determine the system profile"))
+
+
+class VariableClProfileName(ReadonlyVariable):
+    source_var = "cl_profile_system"
+
+    def get(self):
+        make_profile = self.Get(self.source_var)
+        distro, o, name = make_profile.partition('/profiles/')
+        if not o:
+            return make_profile
+        distro = path.basename(distro)
+        return "%s:%s" % (distro, name)
+
+
 class LinuxDataVars(SimpleDataVars):
-    def __init__(self, systemRoot="/"):
+    class VariableOsArchMachine(ReadonlyVariable):
+        systemRoot = '/'
+
+        def get(self):
+            if path.exists(path.join(self.systemRoot, 'lib64')):
+                return 'x86_64'
+            else:
+                return 'i686'
+
+    def variables(self):
+        os_arch_machine = self.VariableOsArchMachine(systemRoot=self.systemRoot)
+
+        return [env.VariableClMakeProfile(systemRoot=self.systemRoot),
+                VariableClProfileName(systemRoot=self.systemRoot),
+                VariableClProfileSystem(systemRoot=self.systemRoot),
+                env.VariableClChrootPath(),
+                os_arch_machine,
+                VariableOsLinuxShortname(systemRoot=self.systemRoot),
+                VariableOsLinuxName(systemRoot=self.systemRoot),
+                VariableOsLinuxSubname(systemRoot=self.systemRoot),
+                VariableOsLinuxFilesnum(systemRoot=self.systemRoot),
+                env.VariableClEmergeConfig(systemRoot=self.systemRoot),
+                VariableOsLinuxFiles(systemRoot=self.systemRoot),
+                VariableOsLinuxSystem(systemRoot=self.systemRoot),
+                VariableOsLinuxVer(systemRoot=self.systemRoot),
+                VariableOsLinuxBuild(systemRoot=self.systemRoot)]
+
+    def __init__(self, systemRoot="/", cache=None):
         self.systemRoot = systemRoot
-        SimpleDataVars.__init__(self,
-            VariableClMakeProfile(systemRoot=systemRoot),
-            VariableOsLinuxShortname(systemRoot=systemRoot),
-            VariableOsLinuxName(systemRoot=systemRoot),
-            VariableOsLinuxSubname(systemRoot=systemRoot),
-            VariableOsLinuxFilesnum(systemRoot=systemRoot),
-            VariableOsLinuxFiles(systemRoot=systemRoot),
-            VariableOsLinuxSystem(systemRoot=systemRoot),
-            VariableOsLinuxVer(systemRoot=systemRoot),
-            VariableOsLinuxBuild(systemRoot=systemRoot))
-        makeprofile = self.Get('cl_make_profile')
-        if os.path.exists(makeprofile):
-            inifile = path.join(systemRoot, os.path.dirname(makeprofile),
-                                os.readlink(makeprofile))
-            self.flIniFileFrom(inifile)
+        SimpleDataVars.__init__(self, *self.variables())
+        self.init_variables()
+        if cache is None:
+            makeprofile = self.Get('cl_make_profile')
+            if os.path.exists(makeprofile):
+                inifile = path.join(systemRoot, os.path.dirname(makeprofile),
+                                    os.readlink(makeprofile))
+                self.flIniFileFrom(inifile, system_root=systemRoot)
+            self.flCalculateEnv(systemRoot)
+        else:
+            self.cache = cache
+
+    def flCalculateEnv(self, systemRoot):
         for inifile in ("etc/calculate/calculate.env",
                         "etc/calculate/calculate3.env",
                         "etc/calculate/calculate2.env"):
-            self.flIniFileFrom(path.join(systemRoot, inifile))
+            self.flIniFileFrom(path.join(systemRoot, inifile),
+                               system_root=systemRoot)
+
+    def init_variables(self):
+        self['cl_chroot_path'] = self.systemRoot
+        return True
+
+    def change_root(self, dn, new_root='/'):
+        if dn.startswith(self.systemRoot):
+            return pathJoin(new_root, dn[len(self.systemRoot):])
+        return dn
